@@ -11,6 +11,8 @@
 #'
 #' @return a raster object containing the spatial distribution of emissions
 #'
+#' @source Example data from Defense Meteorological Satellite Program (DMSP)
+#'
 #' @export
 #'
 #' @import raster sf
@@ -27,9 +29,6 @@
 #' raster::plot(SP,ylab="Lat",xlab="Lon",
 #'              main ="Spatial Distribution by Lights for Sao Paulo - Brazil")
 #'}
-#'
-#'@source Data avaliable \url{https://www.nesdis.noaa.gov/current-satellite-missions/currently-flying/defense-meteorological-satellite-program}
-#'@details About the DMSP and example data \url{https://en.wikipedia.org/wiki/Defense_Meteorological_Satellite_Program}
 
 areaSource <- function(s,r,grid = NA,name = "",as_frac=FALSE,verbose = TRUE){
   if(verbose){
@@ -43,14 +42,14 @@ areaSource <- function(s,r,grid = NA,name = "",as_frac=FALSE,verbose = TRUE){
     if(grid$map_proj %in% c(1,2,3,6)){
       box   <- grid$r
     }else{
-      col   <- grid$Horizontal[1]
-      rol   <- grid$Horizontal[2]
-      r.lat <- range(grid$Lat)
-      r.lon <- range(grid$Lon)
-      box   <- raster::raster(nrows=rol,ncols=col,
-                              xmn=r.lon[1],xmx=r.lon[2],
-                              ymn=r.lat[1],ymx=r.lat[2],
-                              crs='+proj=longlat')
+      col   <- grid$Horizontal[1] # nocov
+      rol   <- grid$Horizontal[2] # nocov
+      r.lat <- range(grid$Lat)    # nocov
+      r.lon <- range(grid$Lon)    # nocov
+      box   <- raster::raster(nrows=rol,ncols=col,       # nocov
+                              xmn=r.lon[1],xmx=r.lon[2], # nocov
+                              ymn=r.lat[1],ymx=r.lat[2], # nocov
+                              crs='+proj=longlat')       # nocov
     }
 
     sp      <- suppressWarnings(raster::projectRaster(sp,crs = raster::crs(box))) # to the new projection
